@@ -90,19 +90,26 @@ file-like objects:
 
 __ver_major__ = 0
 __ver_minor__ = 3
-__ver_patch__ = 1
+__ver_patch__ = 2
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                               __ver_patch__,__ver_sub__)
 
 
-import unittest
 from StringIO import StringIO
 import urllib2
 import urlparse
 import tempfile
 
 
+class NotReadableError(IOError):
+    pass
+class NotWritableError(IOError):
+    pass
+class NotSeekableError(IOError):
+    pass
+
+
 class FileLikeBase(object):
     """Base class for implementing file-like objects.
     
@@ -179,7 +186,7 @@ class FileLikeBase(object):
         'mode' must be one of "r" or "w", and this function returns False
         if the file-like object has a 'mode' attribute, and it does not
         permit access in that mode.  If there is no 'mode' attribute,
-        True is returned.
+        it defaults to "r+".
 
         If seek support is not required, use "r-" or "w-" as the mode string.
 
@@ -190,7 +197,7 @@ class FileLikeBase(object):
             try:
                 mstr = self.mode
             except AttributeError:
-                return True
+                mstr = "r+"
         if "+" in mstr:
             return True
         if "-" in mstr and "-" not in mode:
@@ -213,17 +220,17 @@ class FileLikeBase(object):
             try:
                 mstr = self.mode
             except AttributeError:
-                return True
+                mstr = "r+"
         if "+" in mstr:
             return True
         if "-" in mstr and "-" not in mode:
-            raise IOError("File does not support seeking.")
+            raise NotSeekableError("File does not support seeking.")
         if "r" in mode:
             if "r" not in mstr:
-                raise IOError("File not opened for reading")
+                raise NotReadableError("File not opened for reading")
         if "w" in mode:
             if "w" not in mstr and "a" not in mstr:
-                raise IOError("File not opened for writing")
+                raise NotWritableError("File not opened for writing")
         return True
     
     def flush(self):
@@ -251,7 +258,8 @@ class FileLikeBase(object):
             self.closed = True
 
     def __del__(self):
-        self.close()
+        if not getattr(self,"closed",True):
+            self.close()
 
     def __enter__(self):
         return self
@@ -278,8 +286,8 @@ class FileLikeBase(object):
         """Move the internal file pointer to the given location."""
         if whence > 2 or whence < 0:
             raise ValueError("Invalid value for 'whence': " + str(whence))
-        if hasattr(self,"mode") and "-" in self.mode:
-            raise IOError("File is not seekable.")
+        if "-" in getattr(self,"mode",""):
+            raise NotSeekableError("File is not seekable.")
         # Ensure that there's nothing left in the write buffer
         if self._wbuffer:
             self.flush()
@@ -298,31 +306,35 @@ class FileLikeBase(object):
         # Shortcut the special case of staying put
         if offset == 0 and whence == 1:
             return
-        # Try to do a whence-wise seek if it is implemented.
-        sbuf = None
+        # Catch any failed attempts to read while simulating seek
         try:
-            sbuf = self._seek(offset,whence)
-        except NotImplementedError:
-            # Try to simulate using an absolute seek.
+            # Try to do a whence-wise seek if it is implemented.
+            sbuf = None
             try:
-                if whence == 1:
-                    offset = self._tell() + offset
-                elif whence == 2:
-                    if hasattr(self,"size"):
-                        offset = self.size + offset
-                    else:
-                        for ln in self: pass
-                        offset = self.tell() + offset
-                else:
-                    # absolute seek already failed, don't try again
-                    raise NotImplementedError
-                sbuf = self._seek(offset,0)
+                sbuf = self._seek(offset,whence)
             except NotImplementedError:
-                # Simulate by reseting to start
-                self._seek(0,0)
-                self._soffset = offset
-        finally:
-            self._sbuffer = sbuf
+                # Try to simulate using an absolute seek.
+                try:
+                    if whence == 1:
+                        offset = self._tell() + offset
+                    elif whence == 2:
+                        if hasattr(self,"size"):
+                            offset = self.size + offset
+                        else:
+                            self._do_read_rest()
+                            offset = self.tell() + offset
+                    else:
+                        # absolute seek already failed, don't try again
+                        raise NotImplementedError
+                    sbuf = self._seek(offset,0)
+                except NotImplementedError:
+                    # Simulate by reseting to start
+                    self._seek(0,0)
+                    self._soffset = offset
+            finally:
+                self._sbuffer = sbuf
+        except NotReadableError:
+            raise NotSeekableError("File not readable, can't simulate seek")
 
     def tell(self):
         """Determine current position of internal file pointer."""
@@ -348,6 +360,20 @@ class FileLikeBase(object):
         if self.closed:
             raise IOError("File has been closed")
         self._assert_mode("r-")
+        return self._do_read(size)
+
+    def _do_read(self,size):
+        """Private method to read from the file.
+
+        This method behaves the same as self.read(), but skips some
+        permission and sanity checks.  It is intended for use in simulating
+        seek(), where we may want to read (and discard) information from
+        a file not opened in read mode.
+
+        Note that this may still fail if the file object actually can't
+        be read from - it just won't check whether the mode string gives
+        permission.
+        """
         # If we were previously writing, ensure position is correct
         if self._wbuffer is not None:
             self.seek(0,1)
@@ -360,9 +386,9 @@ class FileLikeBase(object):
             s = self._soffset
             self._soffset = 0
             while s > self._bufsize:
-                self.read(self._bufsize)
+                self._do_read(self._bufsize)
                 s -= self._bufsize
-            self.read(s)
+            self._do_read(s)
         # Should the entire file be read?
         if size <= 0:
             if self._rbuffer:
@@ -399,6 +425,12 @@ class FileLikeBase(object):
                 self._rbuffer = ""
             output = data
         return output
+
+    def _do_read_rest(self):
+        """Private method to read the file through to EOF."""
+        data = self._do_read(self._bufsize)
+        while data != "":
+            data = self._do_read(self._bufsize)
         
     def readline(self,size=-1):
         """Read a line from the file, or at most <size> bytes."""
@@ -455,7 +487,10 @@ class FileLikeBase(object):
         elif self._soffset:
             s = self._soffset
             self._soffset = 0
-            string = self.read(s) + string
+            try:
+                string = self._do_read(s) + string
+            except NotReadableError:
+                raise NotSeekableError("File not readable, could not complete simulation of seek")
             self.seek(0,0)
         if self._wbuffer:
             string = self._wbuffer + string
@@ -488,7 +523,7 @@ class FileLikeBase(object):
         until None has been read from _read().  Once EOF is reached, it
         should be safe to call _read() again, immediately returning None.
         """
-        raise IOError("Object not readable")
+        raise NotReadableError("Object not readable")
     
     def _write(self,string,flushing=False):
         """Write the given string to the file-like object.
@@ -504,7 +539,7 @@ class FileLikeBase(object):
         is expected to be written to the file. If unwritten data is returned
         when 'flushing' is true, an IOError will be raised.
         """
-        raise IOError("Object not writable")
+        raise NotWritableError("Object not writable")
 
     def _seek(self,offset,whence):
         """Set the file's internal position pointer, approximately.
@@ -521,7 +556,7 @@ class FileLikeBase(object):
         NotImplementedError to have them simulated (inefficiently) by
         the higher-level mahinery of this class.
         """
-        raise IOError("Object not seekable")
+        raise NotSeekableError("Object not seekable")
 
     def _tell(self):
         """Get the location of the file's internal position pointer.
@@ -534,7 +569,7 @@ class FileLikeBase(object):
         (the "apparent position") may be different to the position
         returned by this method (the "actual position").
         """
-        raise IOError("Object not seekable")
+        raise NotSeekableError("Object not seekable")
 
 
 class Opener(object):
@@ -567,10 +602,6 @@ class Opener(object):
         self.decoders = [d for d in decoders]
     
     def __call__(self,filename,mode="r"):
-        # Validate the mode string
-        for c in mode:
-            if c not in ("r","w","a",):
-                raise ValueError("Unexpected mode character: '%s'" % (c,))
         # Open the file
         for o in self.openers:
             try:
@@ -597,7 +628,7 @@ class Opener(object):
 
 ##  Create default Opener that uses urllib2.urlopen() and file() as openers
 def _urllib_opener(filename,mode):
-    if mode != "r":
+    if mode not in ("r","r-"):
         return None
     comps = urlparse.urlparse(filename)
     # ensure it's a URL
@@ -849,269 +880,5 @@ def to_filelike(obj,mode="r+"):
     # TODO: lots more could be done here...
     raise ValueError("Could not make object file-like: %s", (obj,))
 
-
-##  Testcases start here
-
-
-class Test_Read(unittest.TestCase):
-    """Generic file-like testcases for readable files."""
-
-    contents = "Once upon a time, in a galaxy far away,\nGuido van Rossum was a space alien."
-
-    def makeFile(self,contents,mode):
-        """This method must create a file of the type to be tested."""
-        return None
-
-    def setUp(self):
-        self.file = self.makeFile(self.contents,"r")
-
-    def tearDown(self):
-        self.file.close()
-
-    def test_read_all(self):
-        c = self.file.read()
-        self.assertEquals(c,self.contents)
-
-    def test_read_size(self):
-        c = self.file.read(5)
-        self.assertEquals(c,self.contents[:5])
-        c = self.file.read(7)
-        self.assertEquals(c,self.contents[5:12])
-
-    def test_readline(self):
-        c = self.file.readline()
-        if self.contents.find("\n") < 0:
-            extra = ""
-        else:
-            extra = "\n"
-        self.assertEquals(c,self.contents.split("\n")[0]+extra)
-
-    def test_readlines(self):
-        cs = [ln.strip("\n") for ln in self.file.readlines()]
-        self.assertEquals(cs,self.contents.split("\n"))
-
-    def test_xreadlines(self):
-        cs = [ln.strip("\n") for ln in self.file.xreadlines()]
-        self.assertEquals(cs,self.contents.split("\n"))
-
-    def test_read_empty_file(self):
-        f = self.makeFile("","r")
-        self.assertEquals(f.read(),"")
-
-    def test_eof(self):
-        self.file.read()
-        self.assertEquals(self.file.read(),"")
-        self.assertEquals(self.file.read(),"")
-
-
-class Test_ReadWrite(Test_Read):
-    """Generic file-like testcases for writable files."""
-
-    def setUp(self):
-        self.file = self.makeFile(self.contents,"r+")
-
-    def test_write_read(self):
-        self.file.write("hello")
-        c = self.file.read()
-        self.assertEquals(c,self.contents[5:])
-
-    def test_read_write_read(self):
-        c = self.file.read(5)
-        self.assertEquals(c,self.contents[:5])
-        self.file.write("hello")
-        c = self.file.read(5)
-        self.assertEquals(c,self.contents[10:15])
-
-
-class Test_ReadWriteSeek(Test_ReadWrite):
-    """Generic file-like testcases for seekable files."""
-
-    def test_seek_tell(self):
-        self.assertEquals(self.file.tell(),0)
-        self.file.seek(7)
-        self.assertEquals(self.file.tell(),7)
-        self.assertEquals(self.file.read(),self.contents[7:])
-        self.file.seek(0,0)
-        self.assertEquals(self.file.tell(),0)
-
-    def test_read_write_seek(self):
-        c = self.file.read(5)
-        self.assertEquals(c,self.contents[:5])
-        self.file.write("hello")
-        self.file.seek(0)
-        self.assertEquals(self.file.tell(),0)
-        c = self.file.read(10)
-        self.assertEquals(c,self.contents[:5] + "hello")
-
-    def test_seek_cur(self):
-        self.assertEquals(self.file.tell(),0)
-        self.file.seek(7,1)
-        self.assertEquals(self.file.tell(),7)
-        self.file.seek(7,1)
-        self.assertEquals(self.file.tell(),14)
-        self.file.seek(-5,1)
-        self.assertEquals(self.file.tell(),9)
-
-    def test_seek_end(self):
-        self.assertEquals(self.file.tell(),0)
-        self.file.seek(-7,2)
-        self.assertEquals(self.file.tell(),len(self.contents)-7)
-        self.file.seek(3,1)
-        self.assertEquals(self.file.tell(),len(self.contents)-4)
-
-
-class Test_StringIO(Test_ReadWriteSeek):
-    """Run our testcases against StringIO."""
-
-    def makeFile(self,contents,mode):
-        f = StringIO(contents)
-        f.seek(0)
-        def xreadlines():
-            for ln in f.readlines():
-                yield ln
-        f.xreadlines = xreadlines
-        return f
-
-
-class Test_TempFile(Test_ReadWriteSeek):
-    """Run our testcases against tempfile.TemporaryFile."""
-
-    def makeFile(self,contents,mode):
-        f = tempfile.TemporaryFile()
-        f.write(contents)
-        f.seek(0)
-        return f
-
-
-class Test_Join(Test_ReadWriteSeek):
-    """Run our testcases against filelike.join."""
-
-    def makeFile(self,contents,mode):
-        files = []
-        files.append(StringIO(contents[0:5]))
-        files.append(StringIO(contents[5:8]))
-        files.append(StringIO(contents[8:]))
-        return join(files)
-
-
-class Test_IsTo(unittest.TestCase):
-    """Tests for is_filelike/to_filelike."""
-
-    def test_isfilelike(self):
-        """Test behaviour of is_filelike."""
-        self.assert_(is_filelike(tempfile.TemporaryFile()))
-        self.assert_(is_filelike(tempfile.TemporaryFile("r"),"r"))
-        self.assert_(is_filelike(tempfile.TemporaryFile("r"),"w"))
-        self.assert_(is_filelike(StringIO()))
-
-    def test_tofilelike_read(self):
-        """Test behavior of to_filelike for mode "r-"."""
-        class F:
-            def read(self,sz=-1):
-                return ""
-        f = to_filelike(F(),"r-")
-        self.assertEquals(f.__class__,wrappers.FileWrapper)
-        self.assertEquals(f.read(),"")
-        self.assertRaises(ValueError,to_filelike,F(),"r")
-        self.assertRaises(ValueError,to_filelike,F(),"w-")
-        self.assertRaises(ValueError,to_filelike,F(),"rw")
-
-    def test_tofilelike_readseek(self):
-        """Test behavior of to_filelike for mode "r"."""
-        class F:
-            def read(self,sz=-1):
-                return ""
-            def seek(self,offset,whence):
-                pass
-        f = to_filelike(F(),"r")
-        self.assertEquals(f.__class__,wrappers.FileWrapper)
-        self.assertEquals(f.read(),"")
-        self.assertRaises(ValueError,to_filelike,F(),"w")
-        self.assertRaises(ValueError,to_filelike,F(),"w-")
-        self.assertRaises(ValueError,to_filelike,F(),"rw")
-
-    def test_tofilelike_write(self):
-        """Test behavior of to_filelike for mode "w-"."""
-        class F:
-            def write(self,data):
-                pass
-        f = to_filelike(F(),"w-")
-        self.assertEquals(f.__class__,wrappers.FileWrapper)
-        self.assertRaises(ValueError,to_filelike,F(),"w")
-        self.assertRaises(ValueError,to_filelike,F(),"r")
-        self.assertRaises(ValueError,to_filelike,F(),"r-")
-        self.assertRaises(ValueError,to_filelike,F(),"rw")
-
-    def test_tofilelike_writeseek(self):
-        """Test behavior of to_filelike for mode "w"."""
-        class F:
-            def write(self,data):
-                pass
-            def seek(self,offset,whence):
-                pass
-        f = to_filelike(F(),"w")
-        self.assertEquals(f.__class__,wrappers.FileWrapper)
-        self.assertRaises(ValueError,to_filelike,F(),"r")
-        self.assertRaises(ValueError,to_filelike,F(),"r-")
-
-    def test_tofilelike_readwrite(self):
-        """Test behavior of to_filelike for mode "rw"."""
-        class F:
-            def write(self,data):
-                pass
-            def read(self,sz=-1):
-                return ""
-            def seek(self,offset,whence):
-                pass
-        f = to_filelike(F(),"rw")
-        self.assertEquals(f.__class__,wrappers.FileWrapper)
-        self.assertEquals(f.read(),"")
-
-    def test_tofilelike_stringio(self):
-        """Test behaviour of to_filelike on StringIO instances."""
-        f = to_filelike(StringIO())
-        self.assert_(isinstance(f,StringIO))
-
-    def test_tofilelike_string(self):
-        """Test behaviour of to_filelike on strings."""
-        f = to_filelike("testing")
-        self.assert_(isinstance(f,StringIO))
-        self.assertEquals(f.read(),"testing")
-        
-
-class Test_Docs(unittest.TestCase):
-    """Unittests for our documentation."""
-
-    def test_readme(self):
-        """Check that README.txt is up-to-date."""
-        import os
-        import difflib
-        readme = os.path.join(os.path.dirname(__file__),"..","README.txt")
-        if os.path.exists(readme):
-            diff = difflib.unified_diff(open(readme).readlines(),__doc__.splitlines(True))
-            diff = "".join(diff)
-            if diff:
-                print diff
-                raise RuntimeError
-
-
-# Included here to avoid circular includes
+# Imported here to aoid circular imports
 import filelike.wrappers
-
-def testsuite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(Test_StringIO))
-    suite.addTest(unittest.makeSuite(Test_TempFile))
-    suite.addTest(unittest.makeSuite(Test_Join))
-    suite.addTest(unittest.makeSuite(Test_IsTo))
-    from filelike import wrappers
-    suite.addTest(wrappers.testsuite())
-    from filelike import pipeline
-    suite.addTest(pipeline.testsuite())
-    suite.addTest(unittest.makeSuite(Test_Docs))
-    return suite
-
-# Run regression tests when called from comand-line
-if __name__ == "__main__":
-    unittest.TextTestRunner().run(testsuite())
-
